@@ -117,15 +117,16 @@ static getdns_dict *listen_dict = NULL;
 static getdns_list *listen_list = NULL;
 static size_t listen_count = 0;
 static int run_in_foreground = 1;
+static int dnssec_validation = 0;
 
 static void stubby_local_log(void *userarg, uint64_t system,
 	getdns_loglevel_type level, const char *fmt, ...);
 
 
 void
-print_usage(FILE *out, const char *progname)
+print_usage(FILE *out)
 {
-	fprintf(out, "usage: %s [<option> ...] \\\n", progname);
+	fprintf(out, "usage: " STUBBY_PACKAGE " [<option> ...] \\\n");
 	fprintf(out, "\t-C\t<filename>\n");
 	fprintf(out, "\t\tRead settings from config file <filename>\n");
 	fprintf(out, "\t\tThe getdns context will be configured with these settings\n");
@@ -154,9 +155,18 @@ print_usage(FILE *out, const char *progname)
 	fprintf(out, "\t\t\t5: NOTICE - %s\n", GETDNS_LOG_NOTICE_TEXT);
 	fprintf(out, "\t\t\t6: INFO   - %s\n", GETDNS_LOG_INFO_TEXT);
 	fprintf(out, "\t\t\t7: DEBUG  - %s\n", GETDNS_LOG_DEBUG_TEXT);
+	fprintf(out, "\t-V\tPrint the " STUBBY_PACKAGE " version\n");
 }
 
+void
+print_version(FILE *out)
+{
+	fprintf(out, STUBBY_PACKAGE_STRING "\n");
+}
+
+#ifndef GETDNS_RETURN_IO_ERROR
 #define GETDNS_RETURN_IO_ERROR ((getdns_return_t) 3000)
+#endif
 
 static const char *_getdns_strerror(getdns_return_t r)
 {
@@ -267,7 +277,8 @@ static getdns_return_t parse_config_file(const char *fn)
 	}
 	config_file[read_sz] = 0;
 	fclose(fh);
-	r = parse_config(config_file, strstr(fn, ".yml") != NULL);
+	r = parse_config(config_file, strstr(fn, ".yml") != NULL
+	                           || strstr(fn, ".yaml") != NULL);
 	free(config_file);
 	if (r == GETDNS_RETURN_GOOD)
 		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
@@ -361,7 +372,7 @@ static void request_cb(
 	dns_msg *msg = (dns_msg *)userarg;
 	uint32_t qid;
 	getdns_return_t r = GETDNS_RETURN_GOOD;
-	uint32_t n, rcode, dnssec_status;
+	uint32_t n, rcode, dnssec_status = GETDNS_DNSSEC_INDETERMINATE;
 
 #if defined(SERVER_DEBUG) && SERVER_DEBUG
 	getdns_bindata *qname;
@@ -384,7 +395,7 @@ static void request_cb(
 
 	if (callback_type != GETDNS_CALLBACK_COMPLETE)
 		SERVFAIL("Callback type not complete",
-		    callback_type, msg, &response);
+		    (int)callback_type, msg, &response);
 
 	else if (!response)
 		SERVFAIL("Missing response", 0, msg, &response);
@@ -397,7 +408,7 @@ static void request_cb(
 	    response, "/replies_tree/0/header/rcode", &rcode))
 		SERVFAIL("No reply in replies tree", 0, msg, &response);
 
-	/* ansers when CD or not BOGUS */
+	/* answers when CD or not BOGUS */
 	else if (!msg->cd_bit && !getdns_dict_get_int(
 	    response, "/replies_tree/0/dnssec_status", &dnssec_status)
 	    && dnssec_status == GETDNS_DNSSEC_BOGUS)
@@ -415,8 +426,9 @@ static void request_cb(
 	    (r = _handle_edns0(response, msg->has_edns0)))
 		SERVFAIL("Could not handle EDNS0", r, msg, &response);
 
-	/* AD when (DO or AD) and SECURE */
-	else if ((r = getdns_dict_set_int(response,"/replies_tree/0/header/ad",
+	/* AD when (DO or AD) and SECURE (But only when we perform validation natively) */
+	else if (dnssec_validation &&
+	    (r = getdns_dict_set_int(response,"/replies_tree/0/header/ad",
 	    ((msg->do_bit || msg->ad_bit)
 	    && (  (!msg->cd_bit && dnssec_status == GETDNS_DNSSEC_SECURE)
 	       || ( msg->cd_bit && !getdns_dict_get_int(response,
@@ -424,13 +436,14 @@ static void request_cb(
 	          && dnssec_status == GETDNS_DNSSEC_SECURE ))) ? 1 : 0)))
 		SERVFAIL("Could not set AD bit", r, msg, &response);
 
-	else if (msg->rt == GETDNS_RESOLUTION_STUB)
-		; /* following checks are for RESOLUTION_RECURSING only */
-	
-	else if ((r =  getdns_dict_set_int(
+	else if ((dnssec_validation || msg->rt == GETDNS_RESOLUTION_RECURSING)
+	    && (r =  getdns_dict_set_int(
 	    response, "/replies_tree/0/header/cd", msg->cd_bit)))
 		SERVFAIL("Could not copy CD bit", r, msg, &response);
 
+	else if (msg->rt == GETDNS_RESOLUTION_STUB)
+		; /* following checks are for RESOLUTION_RECURSING only */
+	
 	else if ((r = getdns_dict_get_int(
 	    response, "/replies_tree/0/header/ra", &n)))
 		SERVFAIL("Could not get RA bit from reply", r, msg, &response);
@@ -514,7 +527,7 @@ static void incoming_request_handler(getdns_context *context,
 			(void)getdns_dict_set_dict(qext, "header", header);
 
 	}
-	if (msg->cd_bit)
+	if (msg->cd_bit && dnssec_validation)
 		getdns_dict_set_int(qext, "dnssec_return_all_statuses",
 		    GETDNS_EXTENSION_TRUE);
 
@@ -641,15 +654,11 @@ main(int argc, char **argv)
 	int opt;
 	long log_level = 7; 
 	char *ep;
+	getdns_dict *api_information = NULL;
+	getdns_list *api_info_keys = NULL;
+	getdns_bindata *api_info_key = NULL;
 
-#ifndef USE_WINSOCK
-	char *prg_name = strrchr(argv[0], '/');
-#else
-	char *prg_name = strrchr(argv[0], '\\');
-#endif
-	prg_name = prg_name ? prg_name + 1 : argv[0];
-
-	while ((opt = getopt(argc, argv, "C:ighlv:")) != -1) {
+	while ((opt = getopt(argc, argv, "C:ighlv:V")) != -1) {
 		switch (opt) {
 		case 'C':
 			custom_config_fn = optarg;
@@ -658,7 +667,7 @@ main(int argc, char **argv)
 			run_in_foreground = 0;
 			break;
 		case 'h':
-			print_usage(stdout, prg_name);
+			print_usage(stdout);
 			exit(EXIT_SUCCESS);
 		case 'i':
 			print_api_info = 1;
@@ -677,8 +686,11 @@ main(int argc, char **argv)
 				exit(EXIT_FAILURE);
 			}
 			break;
+                case 'V':
+			print_version(stdout);
+			exit(EXIT_SUCCESS);
 		default:
-			print_usage(stderr, prg_name);
+			print_usage(stderr);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -750,9 +762,38 @@ main(int argc, char **argv)
 		                 "stub resolution only: %s\n", _getdns_strerror(r));
 		exit(EXIT_FAILURE);
 	}
+	if ((api_information = getdns_context_get_api_information(context))
+	    && !dnssec_validation
+	    && !getdns_dict_get_names(api_information, &api_info_keys)) {
+		size_t i;
+		uint32_t value;
+		getdns_dict *all_context;
+
+		for ( i = 0
+		    ; !dnssec_validation &&
+		      !getdns_list_get_bindata(api_info_keys, i, &api_info_key)
+		    ; i++) {
+			if (!strncmp((const char *)api_info_key->data, "dnssec_", 7)
+			   && !getdns_dict_get_int(api_information, (const char *)api_info_key->data, &value)
+			   && value == GETDNS_EXTENSION_TRUE)
+				dnssec_validation = 1;
+		}
+		if (   !dnssec_validation
+		    && !getdns_dict_get_dict(api_information, "all_context"
+		                                            , &all_context)
+		    && !getdns_dict_get_names(all_context, &api_info_keys)) {
+			for ( i = 0
+			    ; !dnssec_validation &&
+			      !getdns_list_get_bindata(api_info_keys, i, &api_info_key)
+			    ; i++) {
+				if (!strncmp((const char *)api_info_key->data, "dnssec_", 7)
+				   && !getdns_dict_get_int(all_context, (const char *)api_info_key->data, &value)
+				   && value == GETDNS_EXTENSION_TRUE)
+					dnssec_validation = 1;
+			}
+		}
+	}
 	if (print_api_info) {
-		getdns_dict *api_information = 
-		    getdns_context_get_api_information(context);
 		char *api_information_str;
 	       
 		if (listen_dict && !getdns_dict_get_list(
@@ -774,7 +815,6 @@ main(int argc, char **argv)
 		    getdns_pretty_print_dict(api_information);
 		fprintf(stdout, "%s\n", api_information_str);
 		free(api_information_str);
-		getdns_dict_destroy(api_information);
 		fprintf(stderr, "Result: Config file syntax is valid.\n");
 	} else if (listen_count && (r = getdns_context_set_listen_addresses(
 	    context, listen_list, NULL, incoming_request_handler)))
@@ -821,16 +861,61 @@ main(int argc, char **argv)
 				        strerror(errno));
 				exit(EXIT_FAILURE);
 			}
-		} else
+		} else {
+#ifdef SIGPIPE
+			(void)signal(SIGPIPE, SIG_IGN);
+#endif
 			getdns_context_run(context);
+		}
 	} else
 #endif
 	{
+		/* Report basic config options which specifically affect privacy and validation*/
+		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+			       "DNSSEC Validation is %s\n", dnssec_validation==1 ? "ON":"OFF");
+		size_t transport_count = 0;
+		getdns_transport_list_t *transport_list;
+		getdns_context_get_dns_transport_list(context, 
+		                                 &transport_count, &transport_list);
+		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+		                "Transport list is:\n");
+		for (size_t i = 0; i < transport_count; i++) {
+			char* transport_name;
+			switch (transport_list[i]) {
+				case GETDNS_TRANSPORT_UDP:
+					transport_name = "UDP";
+					break;
+				case GETDNS_TRANSPORT_TCP:
+					transport_name = "TCP";
+					break;
+				case GETDNS_TRANSPORT_TLS:
+					transport_name = "TLS";
+					break;
+				default:
+					transport_name = "Unknown transport type";
+					break;
+				}
+			stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+			                 "  - %s\n", transport_name);
+		}
+		free(transport_list);
+		getdns_tls_authentication_t auth;
+		getdns_context_get_tls_authentication(context, &auth);
+		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+		                "Privacy Usage Profile is %s\n", 
+		                 auth==GETDNS_AUTHENTICATION_REQUIRED ? 
+		                "Strict (Authentication required)":"Opportunistic");
+		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+			            "(NOTE a Strict Profile only applies when TLS is the ONLY transport!!)\n");
 		stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_DEBUG,
 			       "Starting DAEMON....\n");
+#ifdef SIGPIPE
+		(void)signal(SIGPIPE, SIG_IGN);
+#endif
 		getdns_context_run(context);
 	}
 
+	getdns_dict_destroy(api_information);
 	getdns_context_destroy(context);
 
 	if (listen_list)
